"""Tests for registry module - nodes methods"""
import os
from pygbif import registry

def test_nodes():
    "Basic test of registry.nodes"
    res = registry.nodes()
    assert dict == res.__class__
    assert 2 == len(res)
    assert 100 == len(res['data'])
    assert ['meta', 'data'] == res.keys()

def test_nodes_limit():
    "limit param in registry.nodes"
    res = registry.nodes(limit=5)
    assert dict == res.__class__
    assert 5 == len(res['data'])

def test_nodes_return():
    "data param in registry.nodes"
    res = registry.nodes(data='identifier', uuid="03e816b3-8f58-49ae-bc12-4e18b358d6d9")
    assert dict == res.__class__
    assert 2 == len(res)
    assert 1 == len(res['data'])
    assert 5 == len(res['data'][0])
    assert 'identifier' in res['data'][0].keys()



import requests

class NoResultException(Exception):
    pass

def gbif_search_GET(url, args, **kwargs):
  # if args['geometry'] != None:
  #   if args['geometry'].__class__ == list:
  #     b = args['geometry']
  #     args['geometry'] = geometry.box(b[0], b[1], b[2], b[3]).wkt
  out = requests.get(url, params=args, **kwargs)
  out.raise_for_status()
  stopifnot(out.headers['content-type'])
  return out.json()

def gbif_GET(url, args, **kwargs):
  out = requests.get(url, params=args, **kwargs)
  out.raise_for_status()
  stopifnot(out.headers['content-type'])
  return out.json()

def stopifnot(x):
  if x != 'application/json':
    raise NoResultException("content-type did not = application/json")

gbif_baseurl = "http://api.gbif.org/v1/"



"""Tests for occurrences module - count methods"""
import os
from pygbif import occurrences

brecord_res = [u'LITERATURE',
 u'OBSERVATION',
 u'LIVING_SPECIMEN',
 u'UNKNOWN',
 u'MATERIAL_SAMPLE',
 u'PRESERVED_SPECIMEN',
 u'MACHINE_OBSERVATION',
 u'HUMAN_OBSERVATION',
 u'FOSSIL_SPECIMEN']

year_res = [u'1991',
 u'1990',
 u'1993',
 u'1992',
 u'1995',
 u'1994',
 u'1997',
 u'1996',
 u'1999',
 u'1998',
 u'2000']

def test_count():
    "occurrences.count - basic test"
    res = occurrences.count(taxonKey = 3329049)
    assert int == res.__class__

def test_count_basisofrecord():
    "occurrences.count_basisofrecord - basic test"
    res = occurrences.count_basisofrecord()
    assert dict == res.__class__
    assert 9 == len(res)
    assert brecord_res == res.keys()

def test_count_year():
    "occurrences.count_year - basic test"
    res = occurrences.count_year(year = '1990,2000')
    assert dict == res.__class__
    assert 11 == len(res)
    assert year_res == res.keys()

def test_count_datasets():
    "occurrences.count_datasets - basic test"
    res = occurrences.count_datasets(country = "DE")
    assert dict == res.__class__
    assert unicode == res.keys()[0].__class__

def test_count_countries():
    "occurrences.count_countries - basic test"
    res = occurrences.count_countries(publishingCountry = "DE")
    assert dict == res.__class__
    assert unicode == res.keys()[0].__class__
    assert int == res.values()[0].__class__

def test_count_schema():
    "occurrences.count_schema - basic test"
    res = occurrences.count_schema()
    assert list == res.__class__
    assert dict == res[0].__class__
    assert "dimensions" == res[0].keys()[0]

def test_count_publishingcountries():
    "occurrences.count_publishingcountries - basic test"
    res = occurrences.count_publishingcountries(country = "DE")
    assert dict == res.__class__
    assert unicode == res.keys()[0].__class__
    assert int == res.values()[0].__class__



"""Tests for registry module - datasets methods"""
import os
from pygbif import registry

def test_datasets():
    "registry.datasets - basic test"
    res = registry.datasets()
    assert dict == res.__class__

def test_datasets_limit():
    "registry.datasets - limit param"
    res = registry.datasets(limit=1)
    assert dict == res.__class__
    assert 1 == len(res['data'])

    res = registry.datasets(limit=3)
    assert dict == res.__class__
    assert 3 == len(res['data'])

def test_datasets_type():
    "registry.datasets - type param"
    res = registry.datasets(type="OCCURRENCE")
    vv = [ x['type'] for x in res['data'] ]
    assert dict == res.__class__
    assert 100 == len(res['data'])
    assert 'OCCURRENCE' == list(set(vv))[0]

def test_dataset_metrics():
    "registry.dataset_metrics - basic test"
    res = registry.dataset_metrics(uuid='3f8a1297-3259-4700-91fc-acc4170b27ce')
    assert dict == res.__class__
    assert 19 == len(res)

def test_dataset_metrics_multiple_uuids():
    "registry.dataset_metrics - multiple uuids"
    uuids = ['3f8a1297-3259-4700-91fc-acc4170b27ce', '66dd0960-2d7d-46ee-a491-87b9adcfe7b1']
    res = registry.dataset_metrics(uuids)
    assert list == res.__class__
    assert 2 == len(res)



from ..gbifutils import *

def name_backbone(name, rank=None, kingdom=None, phylum=None, clazz=None,
  order=None, family=None, genus=None, strict=False, verbose=False,
  start=None, limit=100, **kwargs):
  '''
  Lookup names in the GBIF backbone taxonomy.

  :param name: [str] Full scientific name potentially with authorship (required)
  :param rank: [str] The rank given as our rank enum. (optional)
  :param kingdom: [str] If provided default matching will also try to match against this
     if no direct match is found for the name alone. (optional)
  :param phylum: [str] If provided default matching will also try to match against this
     if no direct match is found for the name alone. (optional)
  :param class: [str] If provided default matching will also try to match against this
     if no direct match is found for the name alone. (optional)
  :param order: [str] If provided default matching will also try to match against this
     if no direct match is found for the name alone. (optional)
  :param family: [str] If provided default matching will also try to match against this
     if no direct match is found for the name alone. (optional)
  :param genus: [str] If provided default matching will also try to match against this
     if no direct match is found for the name alone. (optional)
  :param strict: [bool] If True it (fuzzy) matches only the given name, but never a
     taxon in the upper classification (optional)
  :param verbose: [bool] If True show alternative matches considered which had been rejected.

  A list for a single taxon with many slots (with verbose=False - default), or a
  list of length two, first element for the suggested taxon match, and a data.frame
  with alternative name suggestions resulting from fuzzy matching (with verbose=True).

  If you don't get a match GBIF gives back a list of length 3 with slots synonym,
  confidence, and matchType='NONE'.

  reference: http://www.gbif.org/developer/species#searching

  Usage:
  >>> from pygbif import species
  >>> species.name_backbone(name='Helianthus annuus', kingdom='plants')
  >>> species.name_backbone(name='Helianthus', rank='genus', kingdom='plants')
  >>> species.name_backbone(name='Poa', rank='genus', family='Poaceae')
  >>>
  >>> # Verbose - gives back alternatives
  >>> species.name_backbone(name='Helianthus annuus', kingdom='plants', verbose=True)
  >>>
  >>> # Strictness
  >>> species.name_backbone(name='Poa', kingdom='plants', verbose=True, strict=False)
  >>> species.name_backbone(name='Helianthus annuus', kingdom='plants', verbose=True, strict=True)
  >>>
  >>> # Non-existent name
  >>> species.name_backbone(name='Aso')
  >>>
  >>> # Multiple equal matches
  >>> species.name_backbone(name='Oenante')
  '''
  url = gbif_baseurl + 'species/match'
  args = {'name': name, 'rank': rank, 'kingdom': kingdom, 'phylum': phylum,
         'class': clazz, 'order': order, 'family': family, 'genus': genus,
         'strict': strict, 'verbose': verbose, 'offset': start, 'limit': limit}
  tt = gbif_GET(url, args, **kwargs)
  return tt

def name_suggest(q=None, datasetKey=None, rank=None, fields=None, start=None, limit=100, **kwargs):
  '''
  A quick and simple autocomplete service that returns up to 20 name usages by
  doing prefix matching against the scientific name. Results are ordered by relevance.

  References: http://www.gbif.org/developer/species#searching

  :param q: [str] Simple search parameter. The value for this parameter can be a
     simple word or a phrase. Wildcards can be added to the simple word parameters only,
     e.g. q=*puma* (Required)
  :param datasetKey: [str] Filters by the checklist dataset key (a uuid, see examples)
  :param rank: [str] A taxonomic rank. One of class, cultivar, cultivar_group, domain, family,
     form, genus, informal, infrageneric_name, infraorder, infraspecific_name,
     infrasubspecific_name, kingdom, order, phylum, section, series, species, strain, subclass,
     subfamily, subform, subgenus, subkingdom, suborder, subphylum, subsection, subseries,
     subspecies, subtribe, subvariety, superclass, superfamily, superorder, superphylum,
     suprageneric_name, tribe, unranked, or variety.

  :return: A dictionary, of results

  Usage:
  >>> from pygbif import species
  >>> species.name_suggest(q='Puma concolor')
  >>> x = species.name_suggest(q='Puma')
  >>> x['data']
  >>> x['hierarchy']
  >>> species.name_suggest(q='Puma', rank="genus")
  >>> species.name_suggest(q='Puma', rank="subspecies")
  >>> species.name_suggest(q='Puma', rank="species")
  >>> species.name_suggest(q='Puma', rank="infraspecific_name")
  >>> species.name_suggest(q='Puma', limit=2)
  '''
  url = gbif_baseurl + 'species/suggest'
  args = {'q':q, 'rank':rank, 'offset':start, 'limit':limit}
  tt = gbif_GET(url, args, **kwargs)
  hier = [ x['higherClassificationMap'] for x in tt ]
  [ x.pop('higherClassificationMap') for x in tt ]
  return {'data': tt, 'hierarchy': hier}

def suggestfields():
  '''
  Fields available in `gbif_suggest()` function
  '''
  return ["key","datasetTitle","datasetKey","nubKey","parentKey","parent",
    "kingdom","phylum","class","order","family","genus","species",
    "kingdomKey","phylumKey","classKey","orderKey","familyKey","genusKey",
    "speciesKey","species","canonicalName","authorship",
    "accordingTo","nameType","taxonomicStatus","rank","numDescendants",
    "numOccurrences","sourceId","nomenclaturalStatus","threatStatuses",
    "synonym","higherClassificationMap"]


if __name__ == "__main__":
    import doctest
    doctest.testmod()



from ..gbifutils import *
from .registry_utils import *

def dataset_metrics(uuid, **kwargs):
	'''
	Get details on a GBIF dataset.

	:param uuid: (character) One or more dataset UUIDs. See examples.
	:param **kwargs: Further named parameters.

	References: http://www.gbif.org/developer/registry#datasetMetrics

	Usage:
	>>> from pygbif import registry
	>>> registry.dataset_metrics(uuid='3f8a1297-3259-4700-91fc-acc4170b27ce')
	>>> registry.dataset_metrics(uuid='66dd0960-2d7d-46ee-a491-87b9adcfe7b1')
	>>> registry.dataset_metrics(uuid=['3f8a1297-3259-4700-91fc-acc4170b27ce', '66dd0960-2d7d-46ee-a491-87b9adcfe7b1'])
	'''
	def getdata(x, **kwargs):
		url = gbif_baseurl + 'dataset/' + x + '/metrics'
		return gbif_GET(url, {}, **kwargs)

	if len2(uuid) == 1:
		return getdata(uuid)
	else:
		return [getdata(x) for x in uuid]

def datasets(data = 'all', type = None, uuid = None, query = None, id = None,
							limit = 100, start = None, **kwargs):
	'''
	Search for datasets and dataset metadata.

	:param data: The type of data to get. Default is all data.
	:param type: Type of dataset, options include OCCURRENCE, etc.
	:param uuid: UUID of the data node provider. This must be specified if data
		 is anything other than 'all'.
	:param query: Query term(s). Only used when data='all'
	:param id: A metadata document id.

	References http://www.gbif.org/developer/registry#datasets

	Usage:
	>>> from pygbif import registry
	>>> registry.datasets(limit=5)
	>>> registry.datasets(type="OCCURRENCE")
	>>> registry.datasets(uuid="a6998220-7e3a-485d-9cd6-73076bd85657")
	>>> registry.datasets(data='contact', uuid="a6998220-7e3a-485d-9cd6-73076bd85657")
	>>> registry.datasets(data='metadata', uuid="a6998220-7e3a-485d-9cd6-73076bd85657")
	>>> registry.datasets(data='metadata', uuid="a6998220-7e3a-485d-9cd6-73076bd85657", id=598)
	>>> registry.datasets(data=['deleted','duplicate'])
	>>> registry.datasets(data=['deleted','duplicate'], limit=1)
	'''
	args = {'q': query, 'type': type, 'limit': limit, 'offset': start}
	data_choices = ['all', 'organization', 'contact', 'endpoint',
									'identifier', 'tag', 'machinetag', 'comment',
									'constituents', 'document', 'metadata', 'deleted',
									'duplicate', 'subDataset', 'withNoEndpoint']
	check_data(data, data_choices)

	def getdata(x, uuid, args, **kwargs):
		if x not in ['all','deleted','duplicate','subDataset','withNoEndpoint'] and uuid is None:
			raise TypeError('You must specify a uuid if data does not equal all and data does not equal of deleted, duplicate, subDataset, or withNoEndpoint')

		if uuid is None:
			if x is 'all':
				url = gbif_baseurl + 'dataset'
			else:
				if id is not None and x is 'metadata':
					url = gbif_baseurl + 'dataset/metadata/' + id + '/document'
				else:
					url = gbif_baseurl + 'dataset/' + x
		else:
			if x is 'all':
				url = gbif_baseurl + 'dataset/' + uuid
			else:
				url = gbif_baseurl + 'dataset/' + uuid + '/' + x

		res = gbif_GET(url, args, **kwargs)
		return {'meta': get_meta(res), 'data': parse_results(res, uuid)}

	# Get data
	if len2(data) ==1:
		return getdata(data, uuid, args, **kwargs)
	else:
		return [getdata(x, uuid, args, **kwargs) for x in data]



"""Tests for occurrences module - search methods"""
import os
from pygbif import occurrences

def test_search():
    "occurrences.search - basic test"
    res = occurrences.search(taxonKey = 3329049)
    assert 'dict' == res.__class__.__name__
    assert 5 == len(res)
    assert [u'count', u'endOfRecords', u'limit', u'results', u'offset'] == res.keys()

def test_search_():
    "occurrences.search - diff taxonKey"
    res = occurrences.search(taxonKey = 252408386)
    assert 'dict' == res.__class__.__name__
    assert 24 == len(res)
    assert 252408386 == res['key']

def test_search_():
    "occurrences.search - diff taxonKey2"
    res = occurrences.search(taxonKey = 1052909293)
    assert 'dict' == res.__class__.__name__
    assert 5 == len(res)



# -*- coding: utf-8 -*-

# pygbif

'''
pygbif library
~~~~~~~~~~~~~~~~~~~~~

pygbif is a Python client for GBIF. Example usage:

>>> # Import entire library
>>> import pygbif
>>> # or import modules as needed
>>> ## occurrences
>>> from pygbif import occurrences
>>> ## species
>>> from pygbif import species
>>> ## registry
>>> from pygbif import registry
>>>
>>> ## use advanced logging
>>> ### setup first
>>> import requests
>>> import logging
>>> import httplib as http_client
>>> http_client.HTTPConnection.debuglevel = 1
>>> logging.basicConfig()
>>> logging.getLogger().setLevel(logging.DEBUG)
>>> requests_log = logging.getLogger("requests.packages.urllib3")
>>> requests_log.setLevel(logging.DEBUG)
>>> requests_log.propagate = True
>>> ### then make request
>>> from pygbif import occurrences
>>> occurrences.search(geometry='POLYGON((30.1 10.1, 10 20, 20 40, 40 40, 30.1 10.1))', limit=20)
'''

from .occurrences import search, get, count
from .species import names
from .registry import datasets, nodes
from .gbifissues import occ_issues_lookup



import codecs
from setuptools import setup
from setuptools import find_packages

with codecs.open('README.md', 'r', 'utf-8') as f:
    readme = f.read()

with codecs.open('CHANGES.txt', 'r', 'utf-8') as f:
    changes = f.read()

long_description = readme + '\n\n' + changes

setup(
  name             = 'pygbif',
	version          = '0.1.1',
	description      = 'Python client for GBIF',
  long_description = long_description,
  author           = 'Scott Chamberlain',
  author_email     = 'myrmecocystus@gmail.com',
  url              = 'http://github.com/sckott/pygbif',
  license          = "MIT",
  packages         = find_packages(exclude=['test-*']),
  install_requires = ['requests>2.7'],
  classifiers      = (
    'Development Status :: 3 - Alpha',
    'Intended Audience :: Science/Research',
    'Topic :: Scientific/Engineering :: Bio-Informatics',
    'Natural Language :: English',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python',
    'Programming Language :: Python :: 2.6',
    'Programming Language :: Python :: 2.7'
	)
)
